'use strict'

let _ = require('lodash')
let mongoose = require('mongoose')
let Bluebird = require('bluebird')
let crypto = require('crypto')
let Jwt = require('app/services/jwt')
let Errors = require('app/errors')
let Schema = mongoose.Schema
let ObjectId = Schema.ObjectId
const RESTRICTED = new Set(['email', 'password', 'salt'])

/**
 * Jedi Master Schema
 */
var MasterSchema = new Schema({
  firstName: {
    type: String,
    required: true
  },
  lastName: String,
  origin: String,
  masters: [{
    type: ObjectId,
    ref: 'Master'
  }],
  apprentices: [{
    type: ObjectId,
    ref: 'Master'
  }],
  skills: [String],
  level: Number,
  email: {
    type: String,
    select: false,
    set: val => val.toLowerCase()
  },
  phoneNumber: String,
  role: {
    type: String,
    default: 'user'
  },
  password: {
    type: String,
    select: false
  },
  salt: {
    type: String,
    default: function () {
      var now = new Date().getTime()
      return crypto
        .createHash('md5')
        .update('hello' + now)
        .digest('hex')
    },
    select: false
  },
  createdOn: {
    type: Date,
    default: Date.now
  },
  updatedOn: {
    type: Date,
    default: Date.now
  }
})

MasterSchema.virtual('token').get(function () {
  let payload = {
    _id: this._id,
    role: this.role
  }

  return Jwt.signToken(payload)
})

MasterSchema.pre('save', function (next) {
  // Update the timestamp
  this.updatedOn = Date.now()

  return next()
})

/**
 * Check if password is valid
 *
 * @param {String} password
 *
 * @return {Boolean}
 */
MasterSchema.methods.validatePassword = function validatePassword (password) {
  password = password || ''

  let pwHash = crypto
    .createHash('sha1')
    .update(password + this.salt)
    .digest('hex')

  return this.password === pwHash
}

/**
 * Set the password for this master
 *
 * @param {String} password
 */
MasterSchema.methods.setPassword = function setPassword (password) {
  this.password = crypto
    .createHash('sha1')
    .update(password + this.salt)
    .digest('hex')

  return this
}

// Convert from 'firstName,lastName' to
// { firstName: 1, lastName: 1}
MasterSchema.statics.reduceFields = function (fields) {
  return fields.split(',').reduce((memo, field) => {
    if (RESTRICTED.has(field)) return memo
    memo[field] = 1
    return memo
  }, {})
}

/**
 * Find a master by their email
 *
 * @param {String} email
 *
 * @return {Promise}
 */
MasterSchema.statics.findByEmail = function findByEmail (email) {
  // We need to have an email to search upon
  if (_.isEmpty(email)) {
    return Bluebird.reject(new Errors.BadData('Missing email'))
  }

  return this
    .findOne({ 'email': email.toLowerCase() })
    .select('+email +password +salt')
    .then(function (master) {
      // If null, we didn't find a match
      return _.isNull(master)
        ? Bluebird.reject(new Errors.NotFound(`No master found with ${email}`))
        : master
    })
}

module.exports = mongoose.model('Master', MasterSchema, 'masters')
